package net.rumati.logging.muffero.appender;

import java.io.File;
import java.nio.charset.Charset;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Deque;
import java.util.LinkedList;
import java.util.Map.Entry;
import java.util.SortedMap;
import java.util.TreeMap;
import net.rumati.logging.muffero.layout.Layout;

public class RollingDatedFileAppender
        extends DatedFileAppender
{
    private final Deque<File> managedFiles = new LinkedList<File>();
    private final int keep;
    private String currentFileName = null;

    public RollingDatedFileAppender(String fileNamePattern, Layout layout, int keep)
    {
        this(fileNamePattern, Charset.forName("UTF-8"), layout, keep);
    }

    public RollingDatedFileAppender(String fileNamePattern, Charset charset, Layout layout, int keep)
    {
        super(fileNamePattern, charset, layout);

        if (keep < 2){
            throw new IllegalArgumentException("Invalid amount of log files to keep");
        }
        this.keep = keep;

        int sidx = fileNamePattern.indexOf("{");
        if (sidx < 0){
            throw new IllegalArgumentException("No date format provided");
        }
        int eidx = fileNamePattern.indexOf("}", sidx);
        if (eidx < sidx){
            throw new IllegalArgumentException("Unterminated date format");
        }
        SimpleDateFormat dateFormat = new SimpleDateFormat(fileNamePattern.substring(sidx + 1, eidx));
        String prefix = fileNamePattern.substring(0, sidx);
        String suffix = fileNamePattern.substring(eidx + 1);

        if (suffix.contains(File.separator) || suffix.contains("/")){
            throw new IllegalArgumentException("Date pattern must to be filename, not directory name");
        }

        File parentDirectory = new File(prefix + "x" + suffix).getParentFile();
        if (parentDirectory == null){
            throw new IllegalArgumentException("Invalid file name pattern");
        }

        SortedMap<Date, File> filesMap = new TreeMap<Date, File>();
        File[] existingFiles = parentDirectory.listFiles();
        if (existingFiles != null){
            for (File f : existingFiles){
                String path = f.getAbsolutePath();
                if (path.length() > prefix.length() + suffix.length() && path.startsWith(prefix) && path.endsWith(suffix)){
                    String dateStr = path.substring(prefix.length(), path.length() - suffix.length());
                    try{
                        Date date = dateFormat.parse(dateStr);
                        filesMap.put(date, f);
                    }catch (ParseException e){
                    }
                }
            }
        }

        for (Entry<Date, File> e : filesMap.entrySet()){
            currentFileName = e.getValue().getAbsolutePath();
            managedFiles.addFirst(e.getValue());
        }
    }

    @Override
    protected synchronized String getOutputFileName()
    {
        String fileName = super.getOutputFileName();
        if (!fileName.equals(currentFileName)){
            currentFileName = fileName;
            managedFiles.addFirst(new File(fileName));
            while (managedFiles.size() > keep){
                File f = managedFiles.removeLast();
                if (!f.delete()){
                    System.err.println("WARNING: unable to delete file: " + f.getAbsolutePath());
                }
            }
        }
        return fileName;
    }
}
