package net.rumati.logging.muffero.appender.filter;

import net.rumati.logging.muffero.LogEvent;
import net.rumati.logging.muffero.Priority;
import net.rumati.logging.muffero.appender.Appender;

public class PriorityFilteredAppender
        implements Appender
{
    private final Appender wrappedAppender;
    private final Priority minimumPriority;

    public PriorityFilteredAppender(Appender wrappedAppender, Priority minimumPriority)
    {
        this.wrappedAppender = wrappedAppender;
        this.minimumPriority = minimumPriority;
    }

    public void append(LogEvent event)
    {
        if (!event.getPriority().isLessCriticalThan(minimumPriority)){
            wrappedAppender.append(event);
        }
    }

    public boolean isPriorityEnabled(String loggerName, Priority priority)
    {
        if (priority.isLessCriticalThan(minimumPriority)){
            return false;
        }
        
        return wrappedAppender.isPriorityEnabled(loggerName, priority);
    }

    public void shutdown()
    {
        wrappedAppender.shutdown();
    }
}
