package net.rumati.logging.muffero.config;

import net.rumati.logging.muffero.appender.Appender;
import java.util.LinkedList;
import java.util.List;
import net.rumati.logging.muffero.Logger;
import net.rumati.logging.muffero.LoggingException;

public abstract class Configurator
{
    private static class ConfiguredAppender
    {
        private final Appender appender;
        private final Logger logger;

        public ConfiguredAppender(Appender appender, Logger logger)
        {
            this.appender = appender;
            this.logger = logger;
        }

        public Appender getAppender()
        {
            return appender;
        }

        public Logger getLogger()
        {
            return logger;
        }
    }

    private final List<ConfiguredAppender> configuredAppenders = new LinkedList<ConfiguredAppender>();

    protected synchronized void addAppender(Logger logger, Appender appender)
    {
        logger.addAppender(appender);
        configuredAppenders.add(new ConfiguredAppender(appender, logger));
    }

    public abstract void configure()
            throws LoggingException;

    public synchronized void deconfigure()
    {
        for (ConfiguredAppender cl : configuredAppenders){
            cl.getLogger().removeAppender(cl.getAppender());
        }
    }
}
